@@ -149,6 +149,7 @@ static FullReplay *CurrentReplay;
 static LogEntry *ReplayStep;
 
 static void AppendLog(LogEntry *log, CFile *dest);
+bool IsReplayGame();
 
 //----------------------------------------------------------------------------
 // Log commands
@@ -176,7 +177,7 @@ static FullReplay *StartReplay(void)
 
 	replay->Comment1 = "Generated by Bos Wars Version " VERSION "";
 	replay->Comment2 = "Visit http://www.boswars.org for more information";
-	replay->Comment3 = "$Id: replay.cpp 9589 2009-05-30 18:09:56Z jim4 $";
+	replay->Comment3 = "$Id: replay.cpp 9766 2010-04-21 08:26:37Z jim4 $";
 
 	if (GameSettings.NetGameType == SettingsSinglePlayerGame) {
 		replay->Type = ReplaySinglePlayer;
@@ -310,9 +311,6 @@ static void PrintLogCommand(LogEntry *log, CFile *dest)
 */
 static void SaveFullLog(CFile *dest)
 {
-	LogEntry *log;
-	int i;
-
 	dest->printf("ReplayLog( {\n");
 	dest->printf("  Comment1 = \"%s\",\n", CurrentReplay->Comment1.c_str());
 	dest->printf("  Comment2 = \"%s\",\n", CurrentReplay->Comment2.c_str());
@@ -324,7 +322,7 @@ static void SaveFullLog(CFile *dest)
 	dest->printf("  Type = %d,\n", CurrentReplay->Type);
 	dest->printf("  LocalPlayer = %d,\n", CurrentReplay->LocalPlayer);
 	dest->printf("  Players = {\n");
-	for (i = 0; i < PlayerMax; ++i) {
+	for (int i = 0; i < PlayerMax; ++i) {
 		if (!CurrentReplay->Players[i].Name.empty()) {
 			dest->printf("\t{ Name = \"%s\",", CurrentReplay->Players[i].Name.c_str());
 		} else {
@@ -348,7 +346,8 @@ static void SaveFullLog(CFile *dest)
 	dest->printf("  Network = { %d, %d, %d }\n",
 		CurrentReplay->Network[0], CurrentReplay->Network[1], CurrentReplay->Network[2]);
 	dest->printf("} )\n");
-	log = CurrentReplay->Commands;
+
+	LogEntry *log = CurrentReplay->Commands;
 	while (log) {
 		PrintLogCommand(log, dest);
 		log = log->Next;
@@ -430,7 +429,6 @@ void CommandLog(const char *action, const CUnit *unit, int flush,
 
 	if (!CurrentReplay) {
 		CurrentReplay = StartReplay();
-
 		SaveFullLog(LogFile);
 	}
 
@@ -488,9 +486,7 @@ static int CclLog(lua_State *l)
 	const char *value;
 
 	LuaCheckArgs(l, 1);
-	if (!lua_istable(l, 1)) {
-		LuaError(l, "incorrect argument");
-	}
+	LuaCheckTable(l, 1);
 
 	Assert(CurrentReplay);
 
@@ -548,14 +544,16 @@ static int CclLog(lua_State *l)
 */
 static int CclReplayLog(lua_State *l)
 {
+	if (CommandLogDisabled && !IsReplayGame()) {
+		return 0;
+	}
+
 	FullReplay *replay;
 	const char *value;
 	int j;
 
 	LuaCheckArgs(l, 1);
-	if (!lua_istable(l, 1)) {
-		LuaError(l, "incorrect argument");
-	}
+	LuaCheckTable(l, 1);
 
 	Assert(CurrentReplay == NULL);
 
@@ -583,16 +581,12 @@ static int CclReplayLog(lua_State *l)
 		} else if (!strcmp(value, "LocalPlayer")) {
 			replay->LocalPlayer = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Players")) {
-			if (!lua_istable(l, -1) || lua_objlen(l, -1) != PlayerMax) {
-				LuaError(l, "incorrect argument");
-			}
+			LuaCheckTableSize(l, -1, PlayerMax);
 			for (j = 0; j < PlayerMax; ++j) {
 				int top;
 
 				lua_rawgeti(l, -1, j + 1);
-				if (!lua_istable(l, -1)) {
-					LuaError(l, "incorrect argument");
-				}
+				LuaCheckTable(l, -1);
 				top = lua_gettop(l);
 				lua_pushnil(l);
 				while (lua_next(l, top) != 0) {
@@ -627,31 +621,15 @@ static int CclReplayLog(lua_State *l)
 		} else if (!strcmp(value, "MapRichness")) {
 			replay->MapRichness = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Engine")) {
-			if (!lua_istable(l, -1) || lua_objlen(l, -1) != 3) {
-				LuaError(l, "incorrect argument");
-			}
-			lua_rawgeti(l, -1, 1);
-			replay->Engine[0] = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 2);
-			replay->Engine[1] = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 3);
-			replay->Engine[2] = LuaToNumber(l, -1);
-			lua_pop(l, 1);
+			LuaCheckTableSize(l, -1, 3);
+			replay->Engine[0] = LuaToNumber(l, -1, 1);
+			replay->Engine[1] = LuaToNumber(l, -1, 2);
+			replay->Engine[2] = LuaToNumber(l, -1, 3);
 		} else if (!strcmp(value, "Network")) {
-			if (!lua_istable(l, -1) || lua_objlen(l, -1) != 3) {
-				LuaError(l, "incorrect argument");
-			}
-			lua_rawgeti(l, -1, 1);
-			replay->Network[0] = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 2);
-			replay->Network[1] = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 3);
-			replay->Network[2] = LuaToNumber(l, -1);
-			lua_pop(l, 1);
+			LuaCheckTableSize(l, -1, 3);
+			replay->Network[0] = LuaToNumber(l, -1, 1);
+			replay->Network[1] = LuaToNumber(l, -1, 2);
+			replay->Network[2] = LuaToNumber(l, -1, 3);
 		} else {
 			LuaError(l, "Unsupported key: %s" _C_ value);
 		}
@@ -663,8 +641,6 @@ static int CclReplayLog(lua_State *l)
 	// Apply CurrentReplay settings.
 	if (!SaveGameLoading) {
 		ApplyReplaySettings();
-	} else {
-		CommandLogDisabled = false;
 	}
 
 	return 0;
@@ -685,7 +661,9 @@ bool IsReplayGame()
 */
 void SaveReplayList(CFile *file)
 {
-	SaveFullLog(file);
+	if (CurrentReplay) {
+		SaveFullLog(file);
+	}
 }
 
 /**
@@ -739,10 +717,10 @@ void CleanReplayLog(void)
 	}
 	ReplayStep = NULL;
 
-// if (DisabledLog) {
+	if (DisabledLog) {
 		CommandLogDisabled = false;
 		DisabledLog = false;
-// }
+	}
 	GameObserve = false;
 	NetPlayers = 0;
 	ReplayGameType = ReplayNone;
