@@ -62,6 +62,7 @@
 #include "guichan/widget.h"
 
 #include <iostream>
+#include <string.h>
 
 #include "guichan/sdl/sdlinput.h"
 #include "SDL.h"
@@ -583,12 +584,17 @@ namespace gcn
     }
     
     void Widget::generateAction()
+    {
+        generateAction(mEventId);
+    }
+
+    void Widget::generateAction(const std::string &eventId)
     {
         ActionListenerIterator iter;
         for (iter = mActionListeners.begin(); iter != mActionListeners.end(); ++iter)
         {
-            (*iter)->action(mEventId);
-        }        
+            (*iter)->action(eventId);
+        }
     }
     
     Font* Widget::getFont() const
