@@ -56,6 +56,7 @@
  * For comments regarding functions please see the header file. 
  */
 
+#include <limits>
 #include "guichan/keyinput.h"
 #include "guichan/mouseinput.h"
 #include "guichan/widgets/textfield.h"
@@ -68,8 +69,8 @@ namespace gcn
     {
         mCaretPosition = 0;
         mXScroll = 0;
-		mSelectStart = 0;
-		mSelectEndOffset = 0;
+        mSelectStart = 0;
+        mMaxLengthBytes = std::numeric_limits<int>::max();
 
         setFocusable(true);
 
@@ -83,10 +84,12 @@ namespace gcn
     {
         mCaretPosition = 0;
         mXScroll = 0;
-		mSelectStart = 0;
-		mSelectEndOffset = 0;
+        mSelectStart = 0;
+        mMaxLengthBytes = std::numeric_limits<int>::max();
 
         mText = text;
+        // std::string::max_size() might exceed std::numeric_limits<int>::max().
+        truncateToMaxLength();
         adjustSize();
         setBorderSize(1);
         
@@ -98,18 +101,16 @@ namespace gcn
 
     void TextField::setText(const std::string& text)
     {
-        if ((int)text.size() < mCaretPosition )
-        {
-            mCaretPosition = text.size();
-        }
-    
-        mText = text;    
+        mText = text;
+        truncateToMaxLength();
+        mSelectStart = mCaretPosition;
+        setDirty(true);
     }
   
     void TextField::draw(Graphics* graphics)
     {
-		Font *font;
-		int x, y;
+        Font *font;
+        int x, y;
         Color faceColor = getBackgroundColor();
         graphics->setColor(faceColor);
         graphics->fillRectangle(Rectangle(0, 0, getWidth(), getHeight()));
@@ -120,31 +121,31 @@ namespace gcn
         }
     
         graphics->setColor(getForegroundColor());
-		font = getFont();
+        font = getFont();
         graphics->setFont(font);
 
-		x = 1 - mXScroll;
-		y = 1;
+        x = 1 - mXScroll;
+        y = 1;
 
-		if (mSelectEndOffset != 0)
-		{
-			unsigned int first;
-			unsigned int len;
-			int selX;
-			int selW;
-			std::string tmpStr;
+        if (mSelectStart != mCaretPosition)
+        {
+            unsigned int first;
+            unsigned int len;
+            int selX;
+            int selW;
+            std::string tmpStr;
 
-			getTextSelectionPositions(&first, &len);
+            getTextSelectionPositions(&first, &len);
 
-			tmpStr = std::string(mText.substr(0, first));
-			selX = font->getWidth(tmpStr);
+            tmpStr = std::string(mText.substr(0, first));
+            selX = font->getWidth(tmpStr);
 
-			tmpStr = std::string(mText.substr(first, len));
-			selW = font->getWidth(tmpStr);
+            tmpStr = std::string(mText.substr(first, len));
+            selW = font->getWidth(tmpStr);
 
-			graphics->setColor(Color(127, 127, 127));
-			graphics->fillRectangle(Rectangle(x + selX, y, selW, font->getHeight()));
-		}
+            graphics->setColor(Color(127, 127, 127));
+            graphics->fillRectangle(Rectangle(x + selX, y, selW, font->getHeight()));
+        }
 
         graphics->drawText(mText, x, y);    
     }
@@ -184,30 +185,29 @@ namespace gcn
         if (hasMouse() && button == MouseInput::LEFT)
         {
             mCaretPosition = getFont()->getStringIndexAt(mText, x + mXScroll);
-			mSelectStart = mCaretPosition;
-			mSelectEndOffset = 0;
+            mSelectStart = mCaretPosition;
             fixScroll();
         }
         else if (hasMouse() && button == MouseInput::MIDDLE)
         {
             std::string str;
             if (GetClipboard(str) >= 0) {
-                for (size_t i = 0; i < str.size(); ++i) {
-                    keyPress(Key(str[i]));
-                }
+                // GetClipboard ensures that the string does not
+                // contain control characters.
+                insertAtCaret(str);
+                fixScroll();
             }
         }
     }
 
-	void TextField::mouseMotion(int x, int y)
-	{
-		if (isDragged() && mClickButton == MouseInput::LEFT)
-		{
-			mCaretPosition = getFont()->getStringIndexAt(mText, x + mXScroll);
-			mSelectEndOffset = mCaretPosition - mSelectStart;
-			setDirty(true);
-		}
-	}
+    void TextField::mouseMotion(int x, int y)
+    {
+        if (isDragged() && mClickButton == MouseInput::LEFT)
+        {
+            mCaretPosition = getFont()->getStringIndexAt(mText, x + mXScroll);
+            setDirty(true);
+        }
+    }
 
     bool TextField::keyPress(const Key& key)
     {
@@ -220,20 +220,14 @@ namespace gcn
         if (key.getValue() == Key::LEFT)
         {
             if (mCaretPosition > 0) {
-                mCaretPosition = UTF8GetPrev(mText, mCaretPosition);
-                if (mCaretPosition < 0) {
+                int newpos = UTF8GetPrev(mText, mCaretPosition);
+                if (newpos < 0) {
                     throw GCN_EXCEPTION("Invalid UTF8.");
                 }
-
-                if (key.isShiftPressed()) {
-                    --mSelectEndOffset;
-                } else {
-                    mSelectStart = mCaretPosition;
-                    mSelectEndOffset = 0;
-                }
-            } else if (!key.isShiftPressed()) {
+                mCaretPosition = newpos;
+            }
+            if (!key.isShiftPressed()) {
                 mSelectStart = mCaretPosition;
-                mSelectEndOffset = 0;
             }
 
             ret = true;
@@ -242,20 +236,14 @@ namespace gcn
         else if (key.getValue() == Key::RIGHT)
         {
             if (mCaretPosition < (int)mText.size()) {
-                mCaretPosition = UTF8GetNext(mText, mCaretPosition);
-                if (mCaretPosition > (int)mText.size()) {
+                int newpos = UTF8GetNext(mText, mCaretPosition);
+                if (newpos > (int)mText.size()) {
                     throw GCN_EXCEPTION("Invalid UTF8.");
                 }
-
-                if (key.isShiftPressed()) {
-                    ++mSelectEndOffset;
-                } else {
-                    mSelectStart = mCaretPosition;
-                    mSelectEndOffset = 0;
-                }
-            } else if (!key.isShiftPressed()) {
+                mCaretPosition = newpos;
+            }
+            if (!key.isShiftPressed()) {
                 mSelectStart = mCaretPosition;
-                mSelectEndOffset = 0;
             }
 
             ret = true;
@@ -263,37 +251,38 @@ namespace gcn
 
         else if (key.getValue() == Key::DELETE )
         {
-			if (selLen > 0) {
-				mText.erase(selFirst, selLen);
-				mCaretPosition = selFirst;
-				mSelectStart = selFirst;
-				mSelectEndOffset = 0;
-			} else if (mCaretPosition < (int)mText.size()) {
-				int newpos = UTF8GetNext(mText, mCaretPosition);
-				if (mCaretPosition > (int)mText.size()) {
-					throw GCN_EXCEPTION("Invalid UTF8.");
-				}
-				mText.erase(mCaretPosition, newpos - mCaretPosition);
-				ret = true;
-			}
+            if (selLen > 0) {
+                mText.erase(selFirst, selLen);
+                mCaretPosition = selFirst;
+                mSelectStart = selFirst;
+            } else if (mCaretPosition < (int)mText.size()) {
+                int newpos = UTF8GetNext(mText, mCaretPosition);
+                if (newpos > (int)mText.size()) {
+                    throw GCN_EXCEPTION("Invalid UTF8.");
+                }
+                mText.erase(mCaretPosition, newpos - mCaretPosition);
+            }
+
+            ret = true;
         }
 
         else if (key.getValue() == Key::BACKSPACE || key.getValue() == 'h' - 'a' + 1)
         {
-			if (selLen > 0) {
-				mText.erase(selFirst, selLen);
-				mCaretPosition = selFirst;
-				mSelectStart = selFirst;
-				mSelectEndOffset = 0;
-			} else if (mCaretPosition > 0) {
-				int newpos = UTF8GetPrev(mText, mCaretPosition);
-				if (mCaretPosition < 0) {
-					throw GCN_EXCEPTION("Invalid UTF8.");
-				}
-				mText.erase(newpos, mCaretPosition - newpos);
-				mCaretPosition = newpos;
-				ret = true;
-			}
+            if (selLen > 0) {
+                mText.erase(selFirst, selLen);
+                mCaretPosition = selFirst;
+                mSelectStart = selFirst;
+            } else if (mCaretPosition > 0) {
+                int newpos = UTF8GetPrev(mText, mCaretPosition);
+                if (newpos < 0) {
+                    throw GCN_EXCEPTION("Invalid UTF8.");
+                }
+                mText.erase(newpos, mCaretPosition - newpos);
+                mCaretPosition = newpos;
+                mSelectStart = newpos;
+            }
+
+            ret = true;
         }
 
         else if (key.getValue() == Key::ENTER)
@@ -304,28 +293,22 @@ namespace gcn
 
         else if (key.getValue() == Key::HOME || key.getValue() == 'a' - 'a' + 1) // ctrl-a
         {
-			if (key.isShiftPressed()) {
-				mSelectEndOffset -= mCaretPosition;
-			} else {
-				mSelectStart = 0;
-				mSelectEndOffset = 0;
-			}
             mCaretPosition = 0;
+            if (!key.isShiftPressed()) {
+                mSelectStart = mCaretPosition;
+            }
 
-			ret = true;
+            ret = true;
         }    
 
         else if (key.getValue() == Key::END || key.getValue() == 'e' - 'a' + 1)  //ctrl-e
         {
-			if (key.isShiftPressed()) {
-				mSelectEndOffset += mText.size() - mCaretPosition;
-			} else {
-				mSelectStart = mText.size();
-				mSelectEndOffset = 0;
-			}
-			mCaretPosition = mText.size();
+            mCaretPosition = mText.size();
+            if (!key.isShiftPressed()) {
+                mSelectStart = mCaretPosition;
+            }
 
-			ret = true;
+            ret = true;
         }    
 
         else if (key.getValue() == 'u' - 'a' + 1) // ctrl-u
@@ -338,37 +321,20 @@ namespace gcn
         {
             std::string str;
 
-			if (selLen > 0) {
-				mText.erase(selFirst, selLen);
-				mCaretPosition = selFirst;
-				mSelectStart = selFirst;
-				mSelectEndOffset = 0;
-			}
-
             if (GetClipboard(str) >= 0) {
-                for (size_t i = 0; i < str.size(); ++i) {
-                    keyPress(Key(str[i]));
-                }
-                ret = true;
+                // GetClipboard ensures that the string does not
+                // contain control characters.
+                insertAtCaret(str);
             }
+
+            // Even if GetClipboard failed, we did recognize the key
+            // and the caller should not treat it as a hot key.
+            ret = true;
         }
 
         else if (key.isCharacter())
         {
-            if (selLen > 0) {
-                mText.erase(selFirst, selLen);
-                mCaretPosition = selFirst;
-                mSelectStart = selFirst;
-                mSelectEndOffset = 0;
-            }
-
-            mText.insert(mCaretPosition,key.toString());
-            mCaretPosition = UTF8GetNext(mText, mCaretPosition);
-            if (mCaretPosition > (int)mText.size()) {
-                throw GCN_EXCEPTION("Invalid UTF8.");
-            }
-			mSelectStart = mCaretPosition;
-			mSelectEndOffset = 0;
+            insertAtCaret(key.toString());
             ret = true;
         }
 
@@ -431,19 +397,19 @@ namespace gcn
         return mCaretPosition;    
     }
 
-	void TextField::getTextSelectionPositions(unsigned int* first, unsigned int* len)
-	{
-		if (mSelectEndOffset < 0)
-		{
-			*first = mSelectStart + mSelectEndOffset;
-			*len = -mSelectEndOffset;
-		}
-		else
-		{
-			*first = mSelectStart;
-			*len = mSelectEndOffset;
-		}
-	}
+    void TextField::getTextSelectionPositions(unsigned int* first, unsigned int* len)
+    {
+        if (mCaretPosition < mSelectStart)
+        {
+            *first = mCaretPosition;
+            *len = mSelectStart - mCaretPosition;
+        }
+        else
+        {
+            *first = mSelectStart;
+            *len = mCaretPosition - mSelectStart;
+        }
+    }
 
     const std::string& TextField::getText() const
     {
@@ -454,4 +420,131 @@ namespace gcn
     {
         fixScroll();
     }
+
+    int TextField::getMaxLengthBytes() const
+    {
+        return mMaxLengthBytes;
+    }
+
+    void TextField::setMaxLengthBytes(int maxLengthBytes)
+    {
+        if (maxLengthBytes < 0)
+        {
+            maxLengthBytes = 0;
+        }
+
+        mMaxLengthBytes = maxLengthBytes;
+        truncateToMaxLength();
+    }
+
+    void TextField::truncateToMaxLength()
+    {
+        bool changedSomething = false;
+
+        // Because we never let mMaxLengthBytes become negative,
+        // the following static_cast cannot wrap around.
+        if (mText.size() > static_cast<unsigned int>(mMaxLengthBytes))
+        {
+            int newLength = UTF8LastCharacterBoundary(mText, mMaxLengthBytes);
+            mText.resize(newLength);
+            changedSomething = true;
+        }
+
+        // Because of mMaxLengthBytes, the following static_casts
+        // cannot overflow.
+        if (mCaretPosition > static_cast<int>(mText.size()))
+        {
+            mCaretPosition = mText.size();
+            changedSomething = true;
+        }
+        if (mSelectStart > static_cast<int>(mText.size()))
+        {
+            mSelectStart = mText.size();
+            changedSomething = true;
+        }
+
+        if (changedSomething)
+        {
+            fixScroll();
+            setDirty(true);
+        }
+    }
+
+    int TextField::UTF8LastCharacterBoundary(const std::string &str,
+                                             int maxBytes)
+    {
+        if (maxBytes < 0)
+        {
+            // A bug in the caller.
+            maxBytes = 0;
+        }
+
+        // The following static_cast cannot wrap around because
+        // maxBytes cannot be negative at this point.
+        if (str.size() < static_cast<unsigned int>(maxBytes))
+        {
+            // The following static_cast cannot overflow because
+            // str.size() is less than int maxBytes.
+            maxBytes = static_cast<int>(str.size());
+        }
+
+        if (maxBytes == 0)
+        {
+            // There is no space for any character.
+            return 0;
+        }
+
+        // The following would be easy to implement as:
+        //   return UTF8GetPrev(str, maxBytes + 1);
+        // but maxBytes + 1 might overflow, so don't do that.
+
+        // Find the previous character boundary.
+        int prev = UTF8GetPrev(str, maxBytes);
+        // Assert(prev < maxBytes);
+
+        // If maxBytes already is at a character
+        // boundary, then that character will fit.
+        int next = UTF8GetNext(str, prev);
+        if (next == maxBytes)
+        {
+            return maxBytes;
+        }
+        else
+        {
+            return prev;
+        }
+    }
+
+    void TextField::insertAtCaret(const std::string& str)
+    {
+        int bytesFree = mMaxLengthBytes - mText.size();
+        if (bytesFree < 0)
+        {
+            bytesFree = 0;
+        }
+
+        int bytesToCopy;
+        if (str.size() > static_cast<unsigned int>(bytesFree))
+        {
+            bytesToCopy = UTF8LastCharacterBoundary(str, bytesFree);
+        }
+        else
+        {
+            bytesToCopy = static_cast<int>(str.size());
+        }
+
+        unsigned int selFirst;
+        unsigned int selLen;
+        getTextSelectionPositions(&selFirst, &selLen);
+        if (selLen > 0) {
+            mText.erase(selFirst, selLen);
+            mCaretPosition = selFirst;
+            mSelectStart = selFirst;
+        }
+
+        mText.insert(mCaretPosition, str, 0, bytesToCopy);
+        mCaretPosition += bytesToCopy;
+        mSelectStart = mCaretPosition;
+        setDirty(true);
+    }
 }
