@@ -217,18 +217,6 @@
 **    has active sight even when Removed. It's used for Reveal map
 **    type of spells.
 **
-**  CUnitType::LandUnit
-**
-**    Land animated
-**
-**  CUnitType::AirUnit
-**
-**    Air animated
-**
-**  CUnitType::SeaUnit
-**
-**    Sea animated
-**
 **  CUnitType::ExplodeWhenKilled
 **
 **    Death explosion animated
@@ -324,6 +312,20 @@
 **    Movement mask, this value is and'ed to the map field flags, to
 **    see if a unit can enter or placed on the map field.
 **
+**  CUnitType::ExplicitAllowTerrainMask
+**
+**    Flags that the script explicitly wants to clear in MovementMask,
+**    so that units of this type are allowed in those terrains.
+**    This is used only for computing MovementMask, not in any actual
+**    movement checks.
+**
+**  CUnitType::ExplicitForbidTerrainMask
+**
+**    Flags that the script explicitly wants to set in MovementMask,
+**    so that units of this type are forbidden in those terrains.
+**    This is used only for computing MovementMask, not in any actual
+**    movement checks.
+**
 **  CUnitType::Stats[::PlayerMax]
 **
 **    Unit status for each player
@@ -563,6 +565,18 @@ public:
 	CUnitType *RestrictType;
 };
 
+class CBuildRestrictionTerrain : public CBuildRestriction {
+public:
+	CBuildRestrictionTerrain(unsigned fieldFlags, int min, int max)
+		: FieldFlags(fieldFlags), Min(min), Max(max) {}
+	virtual bool Check(const CUnitType *type, int x, int y, CUnit *&ontoptarget) const;
+
+private:
+	unsigned FieldFlags;
+	int Min;
+	int Max;
+};
+
 	/// Base structure of unit-type
 class CUnitType {
 public:
@@ -629,7 +643,6 @@ public:
 #define MouseActionMove      2      /// Move
 #define MouseActionHarvest   3      /// Harvest resources
 #define MouseActionSpellCast 5      /// Cast the first spell known
-#define MouseActionSail      6      /// Sail
 	int Points;                     /// How many points you get for unit
 	int CanTarget;                  /// Which units can it attack
 #define CanTargetLand 1             /// Can attack land units
@@ -638,9 +651,6 @@ public:
 
 	unsigned Flip : 1;              /// Flip image when facing left
 	unsigned Revealer : 1;          /// reveal the fog of war
-	unsigned LandUnit : 1;          /// Land animated
-	unsigned AirUnit : 1;           /// Air animated
-	unsigned SeaUnit : 1;           /// Sea animated
 	unsigned ExplodeWhenKilled : 1; /// Death explosion animated
 	unsigned Building : 1;          /// Building
 	unsigned VisibleUnderFog : 1;   /// Unit is visible under fog of war.
@@ -697,6 +707,8 @@ public:
 
 	unsigned FieldFlags;            /// Unit map field flags
 	unsigned MovementMask;          /// Unit check this map flags for move
+	unsigned ExplicitAllowTerrainMask; /// Flags to clear from MovementMask
+	unsigned ExplicitForbidTerrainMask; /// Flags to set in MovementMask
 
 	/// @todo This stats should? be moved into the player struct
 	CUnitStats Stats[PlayerMax];     /// Unit status for each player
