@@ -34,7 +34,7 @@
 #include "SDL_opengl.h"
 #include "guichan/image.h"
 
-class CFont;
+class CFontFamily;
 
 extern bool UseOpenGL;
 
@@ -81,6 +81,7 @@ public:
 
 	static void Free(CGraphic *g);
 
+	bool LoadGraphicSize(int *w, int *h);
 	void Load();
 	void Flip();
 	void UseDisplayFormat();
@@ -111,9 +112,7 @@ public:
 	GLuint *Textures;          /// Texture names
 	int NumTextures;           /// Number of textures
 
-	friend void MakeFontColorTextures(CFont *font);
-	friend void CleanFonts(void);
-	friend void ReloadFonts(void);
+	friend class CFontFamily;
 };
 
 class CPlayerColorGraphic : public CGraphic
@@ -345,7 +344,7 @@ extern void InitVideo(void);
 extern int VideoValidResolution(int w, int h);
 
 	/// Load graphic from PNG file
-extern int LoadGraphicPNG(CGraphic *g);
+extern int LoadGraphicPNG(CGraphic *g, bool headerOnly);
 
 	/// Make an OpenGL texture
 extern void MakeTexture(CGraphic *graphic);
@@ -415,7 +414,9 @@ extern EventCallback EditorCallbacks; /// Editor callbacks
 
 extern Uint32 ColorBlack;
 extern Uint32 ColorDarkGreen;
+extern Uint32 ColorDarkBlue;
 extern Uint32 ColorBlue;
+extern Uint32 ColorCyan;
 extern Uint32 ColorOrange;
 extern Uint32 ColorWhite;
 extern Uint32 ColorGray;
