@@ -86,12 +86,14 @@ static int gzseek(CFile *file, unsigned offset, int whence)
 **  @param name       File name.
 **  @param openflags  Open read, or write and compression options
 **
-**  @return File Pointer
+**  @return 0 on success, -1 on error.  The value of errno does not
+**          reliably indicate the cause of the error.
 */
 int CFile::open(const char *name, long openflags)
 {
-	char buf[512];
 	const char *openstring;
+	std::string namedotgz;
+	unsigned char buf[2];
 
 	if ((openflags & CL_OPEN_READ) && (openflags & CL_OPEN_WRITE)) {
 		openstring = "rwb";
@@ -105,18 +107,21 @@ int CFile::open(const char *name, long openflags)
 		return -1;
 	}
 
+	namedotgz = name;
+	namedotgz += ".gz";
+
 	cl_type = CLF_TYPE_INVALID;
 
 	if (openflags & CL_OPEN_WRITE) {
 		if ((openflags & CL_WRITE_GZ) &&
-				(cl_gz = gzopen(strcat(strcpy(buf, name), ".gz"), openstring))) {
+				(cl_gz = gzopen(namedotgz.c_str(), openstring))) {
 			cl_type = CLF_TYPE_GZIP;
 		} else if ((cl_plain = fopen(name, openstring))) {
 			cl_type = CLF_TYPE_PLAIN;
 		}
 	} else {
 		if (!(cl_plain = fopen(name, openstring))) { // try plain first
-			if ((cl_gz = gzopen(strcat(strcpy(buf, name), ".gz"), "rb"))) {
+			if ((cl_gz = gzopen(namedotgz.c_str(), "rb"))) {
 				cl_type = CLF_TYPE_GZIP;
 			}
 		} else {
@@ -141,7 +146,6 @@ int CFile::open(const char *name, long openflags)
 	}
 
 	if (cl_type == CLF_TYPE_INVALID) {
-		//fprintf(stderr, "%s in ", buf);
 		return -1;
 	}
 
@@ -150,6 +154,11 @@ int CFile::open(const char *name, long openflags)
 
 /**
 **  CLclose Library file close
+**
+**  @return 0 on success, nonzero on error.  The resulting value of
+**          errno is best not relied on.  Callers that only read
+**          files, rather than write them, typically ignore the result
+**          anyway.
 */
 int CFile::close()
 {
@@ -160,9 +169,14 @@ int CFile::close()
 
 	if ((tp = cl_type) != CLF_TYPE_INVALID) {
 		if (tp == CLF_TYPE_PLAIN) {
+			// On success, returns 0.
+			// On error, returns EOF and sets errno.
 			ret = fclose(cl_plain);
 		}
 		if (tp == CLF_TYPE_GZIP) {
+			// On success, returns Z_OK=0.
+			// On error, returns a zlib error code, and
+			// sets errno if the code is Z_ERRNO=-1.
 			ret = gzclose(cl_gz);
 		}
 	} else {
