@@ -225,7 +225,7 @@ std::string LocalPlayerName;         /// Name of local player
 
 	/// Name, Version, Copyright
 static std::string NameLine =
-	"Bos Wars V" VERSION ", (c) 1998-2010 by the Bos Wars and Stratagus Project.";
+	"Bos Wars V" VERSION ", (c) 1998-2013 by the Bos Wars and Stratagus Project.";
 
 std::string CliMapName;          /// Filename of the map given on the command line
 std::string CompileOptions;          /// Compile options.
@@ -615,22 +615,28 @@ static void Usage(void)
 {
 	PrintHeader();
 	printf(
-		"\n\nUsage: boswars [OPTIONS]\n\
-		\t-c file.lua\tconfiguration start file (default boswars.lua)\n\
-		\t-d datapath\tpath to Bos Wars data\n\
-		\t-h\t\tHelp shows this page\n\
-		\t-l\t\tDisable command log\n\
-		\t-P port\t\tNetwork port to use\n\
-		\t-n server\tNetwork server host preset\n\
-		\t-L lag\t\tNetwork lag in # frames (default 10 = 333ms)\n\
-		\t-U update\tNetwork update rate in # frames (default 5=6x per s)\n\
-		\t-v mode\t\tVideo mode (0=default,1=640x480,2=800x600,\n\
-		\t\t\t\t3=1024x768,4=1280x960,5=1600x1200)\n\
-		\t-D\t\tVideo mode depth = pixel per point (for Win32/TNT)\n\
-		\t-F\t\tFull screen video mode\n\
-		\t-S\t\tSync speed (100 = 30 frames/s)\n\
-		\t-W\t\tWindowed video mode\n\
-		");
+		"\n\n"
+		"Usage: boswars [OPTIONS] [MAP-PRESENTATION-FILE]\n"
+		"  -c file.lua   Configuration start file (default scripts/boswars.lua)\n"
+		"  -d datapath   Path to Bos Wars data\n"
+		"  -E file.lua   Load this script file when starting the map editor\n"
+		"                  (default scripts/editor.lua)\n"
+#ifdef DEBUG
+		"  -e            Show all events received from SDL (requires DEBUG)\n"
+#endif
+		"  -h            Help shows this page\n"
+		"  -l            Disable command log\n"
+		"  -P port       Network port to use\n"
+		"  -L lag        Network lag in # frames (default 10 = 333ms)\n"
+		"  -U update     Network update rate in # frames (default 5=6x per s)\n"
+		"  -v mode       Video mode WIDTHxHEIGHT, at least 640x480.  Abbreviations:\n"
+		"                  1=640x480, 2=800x600, 3=1024x768, 4=1280x960, 5=1600x1200.\n"
+		"  -D bpp        Video mode depth = bits per pixel (for Win32/TNT)\n"
+		"  -F            Full screen video mode\n"
+		"  -S speed      Video sync speed (100 = 30 frames/s)\n"
+		"  -W            Windowed video mode\n"
+		"Specifying MAP-PRESENTATION-FILE starts a single-player game in that map.\n"
+		"Example: boswars maps/islandwar.map/presentation.smp\n");
 }
 
 #ifdef REDIRECT_OUTPUT
@@ -676,6 +682,64 @@ static void RedirectOutput()
 }
 #endif
 
+/**
+**  Parses the "-v mode" command-line option.
+**
+**  @param arg  The "mode" argument of the "-v mode" option.
+*/
+static void VideoModeOption(const char *arg)
+{
+	int abbrev = 0;
+	int width = 0;
+	int height = 0;
+	char garbage;
+
+	if (sscanf(arg, "%d %c", &abbrev, &garbage) == 1) {
+		switch (abbrev) {
+			case 0:
+				Video.Width = 0;
+				Video.Height = 0;
+				return;
+			case 1:
+				Video.Width = 640;
+				Video.Height = 480;
+				return;
+			case 2:
+				Video.Width = 800;
+				Video.Height = 600;
+				return;
+			case 3:
+				Video.Width = 1024;
+				Video.Height = 768;
+				return;
+			case 4:
+				Video.Width = 1280;
+				Video.Height = 960;
+				return;
+			case 5:
+				Video.Width = 1600;
+				Video.Height = 1200;
+				return;
+		}
+	}
+			
+	if (sscanf(arg, "%d%*[Xx]%d %c", &width, &height, &garbage) == 2) {
+		// Too small resolutions can cause segmentation faults
+		// when the engine tries to draw outside the screen.
+		if (width < 640 || height < 480) {
+			fprintf(stderr, "Video mode must be at least 640x480.\n");
+			ExitFatal(-1);
+		}
+
+		Video.Width = width;
+		Video.Height = height;
+		return;
+	} 
+	
+	Usage();
+	ExitFatal(-1);
+}
+
 /**
 **  The main program: initialise, parse options and arguments.
 **
@@ -741,7 +805,7 @@ int main(int argc, char **argv)
 	//  Parse commandline
 	//
 	for (;;) {
-		switch (getopt(argc, argv, "c:d:hln:P:v:D:N:E:FL:S:U:W?")) {
+		switch (getopt(argc, argv, "c:d:ehlP:v:D:E:FL:S:U:W?")) {
 			case 'c':
 				CclStartFile = optarg;
 				continue;
@@ -751,6 +815,23 @@ int main(int argc, char **argv)
 				replace(StratagusLibPath.begin(), StratagusLibPath.end(), '\\', '/');
 				continue;
 			}
+			case 'e':
+				// DumpAllSdlEvents is intended for engine
+				// hackers, rather than scripters or players.
+				// Make it available in DEBUG builds only, so
+				// that we are more free to make incompatible
+				// changes to it, and so that players won't
+				// complain that the feature is cumbersome to
+				// use in Windows or Mac OS X.
+#ifdef DEBUG
+				DumpAllSdlEvents = true;
+#else
+				// The following string is not in _("...")
+				// because the translation files have not
+				// been loaded yet.
+				fprintf(stderr, "Warning: Option -e is available in DEBUG builds only.\n");
+#endif
+				continue;
 			case 'E':
 				EditorStartFile = optarg;
 				continue;
@@ -760,42 +841,9 @@ int main(int argc, char **argv)
 			case 'P':
 				NetworkPort = atoi(optarg);
 				continue;
-			case 'n':
-				NetworkArg = optarg;
-				continue;
-			case 'N':
-				LocalPlayerName = optarg;
-				continue;
 			case 'v':
-				switch (atoi(optarg)) {
-					case 0:
-						continue;
-					case 1:
-						Video.Width = 640;
-						Video.Height = 480;
-						continue;
-					case 2:
-						Video.Width = 800;
-						Video.Height = 600;
-						continue;
-					case 3:
-						Video.Width = 1024;
-						Video.Height = 768;
-						continue;
-					case 4:
-						Video.Width = 1280;
-						Video.Height = 960;
-						continue;
-					case 5:
-						Video.Width = 1600;
-						Video.Height = 1200;
-						continue;
-					default:
-						Usage();
-						ExitFatal(-1);
-				}
+				VideoModeOption(optarg);
 				continue;
-
 			case 'L':
 				NetworkLag = atoi(optarg);
 				if (!NetworkLag) {
