@@ -67,6 +67,11 @@ std::map<std::string, CUnitType *> UnitTypeMap;
 */
 std::string DefaultResourceNames[MaxCosts];
 
+/**
+**  Default names for the resources used for display (localized).
+*/
+std::string DefaultDisplayResourceNames[MaxCosts];
+
 /*----------------------------------------------------------------------------
 --  Functions
 ----------------------------------------------------------------------------*/
@@ -84,7 +89,7 @@ CUnitType::CUnitType() :
 	CanTransport(false), MaxOnBoard(0),
 	UnitType(UnitTypeLand), DecayRate(0), AnnoyComputerFactor(0),
 	MouseAction(0), Points(0), CanTarget(0),
-	Flip(0), Revealer(0), LandUnit(0), AirUnit(0), SeaUnit(0),
+	Flip(0), Revealer(0),
 	ExplodeWhenKilled(0), Building(0), VisibleUnderFog(0),
 	Coward(0), AttackFromTransporter(0),
 	Vanishes(0), GroundAttack(0), ShoreBuilding(0), CanAttack(0),
@@ -92,6 +97,7 @@ CUnitType::CUnitType() :
 	Neutral(0), SelectableByRectangle(0), IsNotSelectable(0), Decoration(0),
 	Indestructible(0), Organic(0), Variable(NULL),
 	ProductionEfficiency(100), FieldFlags(0), MovementMask(0),
+	ExplicitAllowTerrainMask(0), ExplicitForbidTerrainMask(0),
 	Sprite(NULL), ShadowSprite(NULL)
 {
 	memset(&NeutralMinimapColorRGB, 0, sizeof(NeutralMinimapColorRGB));
@@ -163,7 +169,8 @@ void UpdateStats(int reset)
 					MapFieldSeaUnit |
 					MapFieldBuilding | // already occuppied
 					MapFieldCoastAllowed |
-					MapFieldWaterAllowed | // can't move on this
+					MapFieldShallowWater |
+					MapFieldDeepWater | // can't move on this
 					MapFieldUnpassable;
 				break;
 			case UnitTypeFly:                               // in air
@@ -171,20 +178,15 @@ void UpdateStats(int reset)
 					MapFieldAirUnit; // already occuppied
 				break;
 			case UnitTypeNaval:                             // on water
-				if (type->CanTransport) {
-					type->MovementMask =
-						MapFieldLandUnit |
-						MapFieldSeaUnit |
-						MapFieldBuilding | // already occuppied
-						MapFieldLandAllowed; // can't move on this
-					// Johns: MapFieldUnpassable only for land units?
-				} else {
-					type->MovementMask =
-						MapFieldLandUnit |
-						MapFieldSeaUnit |
-						MapFieldBuilding | // already occuppied
+				type->MovementMask =
+					MapFieldLandUnit |
+					MapFieldSeaUnit |
+					MapFieldBuilding | // already occupied
+					MapFieldLandAllowed; // can't move on this
+				// Johns: MapFieldUnpassable only for land units?
+				if (!type->CanTransport) {
+					type->MovementMask |=
 						MapFieldCoastAllowed |
-						MapFieldLandAllowed | // can't move on this
 						MapFieldUnpassable;
 				}
 				break;
@@ -202,7 +204,14 @@ void UpdateStats(int reset)
 					MapFieldBuilding | // already occuppied
 					MapFieldLandAllowed; // can't build on this
 			}
-			type->MovementMask |= MapFieldNoBuilding;
+			// Neutral harvestable units such as rocks and trees
+			// are marked as buildings because they cannot move
+			// and their corpses should not fuel spells.  They
+			// should however ignore MapFieldNoBuilding so that
+			// the map author can place them wherever desired.
+			if (!type->Neutral) {
+				type->MovementMask |= MapFieldNoBuilding;
+			}
 			//
 			// A little chaos, buildings without HP can be entered.
 			// The oil-patch is a very special case.
@@ -227,6 +236,9 @@ void UpdateStats(int reset)
 				type->FieldFlags = 0;
 				break;
 		}
+
+		type->MovementMask &= ~type->ExplicitAllowTerrainMask;
+		type->MovementMask |= type->ExplicitForbidTerrainMask;
 	}
 }
 
@@ -239,7 +251,13 @@ void UpdateStats(int reset)
 */
 CAnimations *AnimationsByIdent(const std::string &ident)
 {
-	return AnimationMap[ident];
+	std::map<std::string, CAnimations *>::const_iterator found
+		= AnimationMap.find(ident);
+	if (found != AnimationMap.end()) {
+		return found->second;
+	} else {
+		return NULL;
+	}
 }
 
 /**
@@ -251,7 +269,13 @@ CAnimations *AnimationsByIdent(const std::string &ident)
 */
 CUnitType *UnitTypeByIdent(const std::string &ident)
 {
-	return UnitTypeMap[ident];
+	std::map<std::string, CUnitType *>::const_iterator found
+		= UnitTypeMap.find(ident);
+	if (found != UnitTypeMap.end()) {
+		return found->second;
+	} else {
+		return NULL;
+	}
 }
 
 /**
