@@ -52,12 +52,47 @@
 --  Variables
 ----------------------------------------------------------------------------*/
 
+static enum {
+	ModuleNotInitialized,
+	ModuleOK,
+	ModuleDeinitialized
+} GraphicsState;
+
 static int HashCount;
 
 static std::map<std::string, CGraphic *> GraphicHash;
 
 static std::list<CGraphic *> Graphics;
 
+struct CGraphicsModule {
+	CGraphicsModule() {
+		GraphicsState = ModuleOK;
+	}
+	~CGraphicsModule() {
+		GraphicsState = ModuleDeinitialized;
+	}
+	static void AssertOK() {
+#ifdef DEBUG
+		switch (GraphicsState) {
+		case ModuleNotInitialized:
+			fprintf(stderr, "The graphics module hasn't yet been initialized\n");
+			// Do not call ExitFatal, because that calls exit,
+			// which might attempt to call destructors again.
+			abort();
+		case ModuleOK:
+			break;
+		case ModuleDeinitialized:
+			fprintf(stderr, "The graphics module has already been deinitialized\n");
+			abort();
+		default:
+			abort();
+		}
+#endif	// DEBUG
+	}
+};
+
+static CGraphicsModule GraphicsModule;
+
 /*----------------------------------------------------------------------------
 --  Functions
 ----------------------------------------------------------------------------*/
@@ -91,8 +126,18 @@ void CGraphic::DrawClip(int x, int y) const
 void CGraphic::DrawSub(int gx, int gy, int w, int h, int x, int y) const
 {
 	if (!UseOpenGL) {
-		SDL_Rect srect = {gx, gy, w, h};
-		SDL_Rect drect = {x, y, 0, 0};
+		SDL_Rect srect = {
+			static_cast<Sint16>(gx),
+			static_cast<Sint16>(gy),
+			static_cast<Uint16>(w),
+			static_cast<Uint16>(h)
+		};
+		SDL_Rect drect = {
+			static_cast<Sint16>(x),
+			static_cast<Sint16>(y),
+			0, // SDL_BlitSurface ignores the width and height.
+			0
+		};
 
 		SDL_BlitSurface(Surface, &srect, TheScreen, &drect);
 	} else {
@@ -198,17 +243,16 @@ void CGraphic::DoDrawFrameClip(GLuint *textures,
 	int skip;
 	int w;
 	int h;
-	int sx, sy, ex, ey, n;
+	int sx, sy, n;
 
 	n = GraphicWidth / Width;
 	sx = (frame % n) * Width;
-	ex = sx + Width;
 	sy = (frame / n) * Height;
-	ey = sy + Height;
 
 	w = Width;
 	h = Height;
 	CLIP_RECTANGLE_OFS(x, y, w, h, ox, oy, skip);
+	skip = skip; // avoid a compiler warning
 
 	DrawTexture(this, textures, sx + ox, sy + oy, sx + ox + w, sy + oy + h, x, y, 0);
 }
@@ -326,17 +370,16 @@ void CGraphic::DoDrawFrameClipX(GLuint *textures, unsigned frame,
 	int skip;
 	int w;
 	int h;
-	int sx, sy, ex, ey, n;
+	int sx, sy, n;
 
 	n = GraphicWidth / Width;
 	sx = (frame % n) * Width;
-	ex = sx + Width;
 	sy = (frame / n) * Height;
-	ey = sy + Height;
 
 	w = Width;
 	h = Height;
 	CLIP_RECTANGLE_OFS(x, y, w, h, ox, oy, skip);
+	skip = skip; // avoid a compiler warning
 
 	if (w < Width) {
 		if (ox == 0) {
@@ -490,6 +533,8 @@ CGraphic *CGraphic::New(const std::string &file, int w, int h)
 		return new CGraphic;
 	}
 
+	CGraphicsModule::AssertOK();
+
 	CGraphic *g = GraphicHash[file];
 	if (!g) {
 		g = new CGraphic;
@@ -526,6 +571,8 @@ CPlayerColorGraphic *CPlayerColorGraphic::New(const std::string &file, int w, in
 		return new CPlayerColorGraphic;
 	}
 
+	CGraphicsModule::AssertOK();
+
 	CPlayerColorGraphic *g = dynamic_cast<CPlayerColorGraphic *>(GraphicHash[file]);
 	if (!g) {
 		g = new CPlayerColorGraphic;
@@ -558,6 +605,7 @@ CPlayerColorGraphic *CPlayerColorGraphic::New(const std::string &file, int w, in
 */
 CGraphic *CGraphic::ForceNew(const std::string &file, int w, int h)
 {
+	CGraphicsModule::AssertOK();
 	CGraphic *g = new CGraphic;
 	if (!g) {
 		fprintf(stderr, "Out of memory\n");
@@ -602,6 +650,7 @@ CGraphic *CGraphic::Clone() const
 */
 CPlayerColorGraphic *CPlayerColorGraphic::ForceNew(const std::string &file, int w, int h)
 {
+	CGraphicsModule::AssertOK();
 	CPlayerColorGraphic *g = new CPlayerColorGraphic;
 	if (!g) {
 		fprintf(stderr, "Out of memory\n");
@@ -620,6 +669,31 @@ CPlayerColorGraphic *CPlayerColorGraphic::ForceNew(const std::string &file, int
 	return g;
 }
 
+/**
+**  Load the size of a graphic, but not the actual pixels.
+**  Also, unlike Load(), this does not abort the whole process
+**  if the file cannot be loaded.
+**
+**  @param[out] w     width of the graphic, in pixels
+**  @param[out] h     height of the graphic, in pixels
+**
+**  @return true on success, or false on error.
+*/
+bool CGraphic::LoadGraphicSize(int *w, int *h)
+{
+	if (GraphicWidth == 0 && GraphicHeight == 0) {
+		if (LoadGraphicPNG(this, true) == -1) {
+			return false;
+		}
+	}
+
+	Assert(GraphicWidth != 0 && GraphicHeight != 0);
+	*w = GraphicWidth;
+	*h = GraphicHeight;
+	return true;
+}
+
+
 /**
 **  Load a graphic
 */
@@ -629,8 +703,10 @@ void CGraphic::Load()
 		return;
 	}
 
+	CGraphicsModule::AssertOK();
+
 	// TODO: More formats?
-	if (LoadGraphicPNG(this) == -1) {
+	if (LoadGraphicPNG(this, false) == -1) {
 		fprintf(stderr, "Can't load the graphic `%s'\n", File.c_str());
 		ExitFatal(-1);
 	}
@@ -713,14 +789,18 @@ void CGraphic::Free(CGraphic *g)
 				}
 			}
 
-			Graphics.remove(g);
+			if (GraphicsState == ModuleOK) {
+				Graphics.remove(g);
+			}
 		}
 
 		FreeSurface(&g->Surface);
 		FreeSurface(&g->SurfaceFlip);
 
 		if (!g->HashFile.empty()) {
-			GraphicHash.erase(g->HashFile);
+			if (GraphicsState == ModuleOK) {
+				GraphicHash.erase(g->HashFile);
+			}
 		}
 		delete g;
 	}
@@ -731,6 +811,8 @@ void CGraphic::Free(CGraphic *g)
 */
 void FreeOpenGLGraphics()
 {
+	CGraphicsModule::AssertOK();
+
 	std::list<CGraphic *>::iterator i;
 	for (i = Graphics.begin(); i != Graphics.end(); ++i) {
 		if ((*i)->Textures) {
@@ -752,6 +834,7 @@ void FreeOpenGLGraphics()
 */
 void ReloadGraphics()
 {
+	CGraphicsModule::AssertOK();
 	std::list<CGraphic *>::iterator i;
 	for (i = Graphics.begin(); i != Graphics.end(); ++i) {
 		if ((*i)->Textures) {
@@ -778,6 +861,8 @@ void ReloadGraphics()
 void CGraphic::Flip()
 {
 	if (UseOpenGL) {
+		// OpenGL mode does not use SurfaceFlip; instead,
+		// it swaps the texture coordinates when drawing.
 		return;
 	}
 
@@ -884,7 +969,6 @@ static void MakeTextures2(CGraphic *g, GLuint texture, CUnitColors *colors,
 	unsigned char *tex;
 	unsigned char *tp;
 	const unsigned char *sp;
-	int fl;
 	Uint32 ckey;
 	int useckey;
 	int bpp;
@@ -896,7 +980,6 @@ static void MakeTextures2(CGraphic *g, GLuint texture, CUnitColors *colors,
 	int maxw;
 	int maxh;
 
-	fl = g->GraphicWidth / g->Width;
 	useckey = g->Surface->flags & SDL_SRCCOLORKEY;
 	f = g->Surface->format;
 	bpp = f->BytesPerPixel;
@@ -1102,6 +1185,12 @@ void CGraphic::Resize(int w, int h)
 		FreeSurface(&Surface);
 		FreeSurface(&SurfaceFlip);
 
+		if (UseOpenGL && Textures) {
+			glDeleteTextures(NumTextures, Textures);
+			delete[] Textures;
+			Textures = NULL;
+		}
+
 		this->Width = this->Height = 0;
 		this->Load();
 
@@ -1212,7 +1301,7 @@ void CGraphic::Resize(int w, int h)
 	Width = GraphicWidth = w;
 	Height = GraphicHeight = h;
 
-	if (UseOpenGL) {
+	if (UseOpenGL && Textures) {
 		glDeleteTextures(NumTextures, Textures);
 		delete[] Textures;
 		Textures = NULL;
@@ -1289,6 +1378,7 @@ void CGraphic::MakeShadow()
 
 void FreeGraphics()
 {
+	CGraphicsModule::AssertOK();
 	std::map<std::string, CGraphic *>::iterator i;
 	while (!GraphicHash.empty()) {
 		i = GraphicHash.begin();
